 /*By Chris Anderson & Jordi Munoz*/
/**ArduPilot Version 2.2.3 Version**/
 
#include <avr/eeprom.h>
#include <avr/io.h>
#include <magpie.h>

// - Load the airframe settings. 


//GPS SIRF configuration strings... 
#define SIRF_BAUD_RATE_4800    "$PSRF100,1,4800,8,1,0*0E\r\n"
#define SIRF_BAUD_RATE_9600    "$PSRF100,1,9600,8,1,0*0D\r\n"
#define SIRF_BAUD_RATE_19200    "$PSRF100,1,19200,8,1,0*38\r\n"
#define SIRF_BAUD_RATE_38400    "$PSRF100,1,38400,8,1,0*3D\r\n"  
#define SIRF_BAUD_RATE_57600    "$PSRF100,1,57600,8,1,0*36\r\n"
#define GSA_ON   "$PSRF103,2,0,1,1*27\r\n"   // enable GSA
#define GSA_OFF  "$PSRF103,2,0,0,1*26\r\n"   // disable GSA
#define GSV_ON   "$PSRF103,3,0,1,1*26\r\n"  // enable GSV
#define GSV_OFF  "$PSRF103,3,0,0,1*27\r\n"  // disable GSV
#define USE_WAAS   1     //1 = Enable, 0 = Disable, good in USA, slower FIX... 
#define WAAS_ON    "$PSRF151,1*3F\r\n"       // enable WAAS
#define WAAS_OFF   "$PSRF151,0*3E\r\n"       // disable WAAS

//GPS Locosys configuration strings...
#define USE_SBAS 0
#define SBAS_ON "$PMTK313,1*2E\r\n"
#define SBAS_OFF "$PMTK313,0*2F\r\n"

#define NMEA_OUTPUT_5HZ "$PMTK314,0,5,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0*28\r\n" //Set GGA and RMC to 5HZ  
#define NMEA_OUTPUT_4HZ "$PMTK314,0,4,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0*28\r\n" //Set GGA and RMC to 4HZ 
#define NMEA_OUTPUT_3HZ "$PMTK314,0,3,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0*28\r\n" //Set GGA and RMC to 3HZ 
#define NMEA_OUTPUT_2HZ "$PMTK314,0,2,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0*28\r\n" //Set GGA and RMC to 2HZ 
#define NMEA_OUTPUT_1HZ "$PMTK314,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0*28\r\n" //Set GGA and RMC to 1HZ

#define LOCOSYS_REFRESH_RATE_200 "$PMTK220,200*2C" //200 milliseconds 
#define LOCOSYS_REFRESH_RATE_250 "$PMTK220,250*29\r\n" //250 milliseconds

#define LOCOSYS_BAUD_RATE_4800 "$PMTK251,4800*14\r\n"
#define LOCOSYS_BAUD_RATE_9600 "$PMTK251,9600*17\r\n"
#define LOCOSYS_BAUD_RATE_19200 "$PMTK251,19200*22\r\n"
#define LOCOSYS_BAUD_RATE_38400 "$PMTK251,38400*27\r\n"

#define Battery_Voltage(x) (x*(INTPUT_VOLTAGE/1024.0))/(100000.0/(240000.0+100000.0))/1000.0 //I explained this in the Voltage Divider Theory! 
#define Pressure_MTS(x) sqrt((((x*(INTPUT_VOLTAGE/1024.0))/1.0)*2.0)/1.225) //Converts to AirSpeed in meters for second, i will later explain this. 
#define Pressure_KMH(x) Pressure_MTS(x)*3.6 //Kilometers per hour, never use English system please. This stuff is international. 

#define PID_dt 20                // Servo refresh time in milliseconds 

#define YELLOW_LED 13
#define BLUE_LED 12

#define GOOD_FIX 0
#define NO_FIX 1
#define FLY_BY_WIRE 4

// ------------------------------------------------------------------------
// EEPROM Addresses for stored values

#define EEPROM_Roll_FBW_Gain   0x00
#define EEPROM_Pitch_FBW_Gain  0x01

#define EEPROM_Roll_Max   0x02
#define EEPROM_Pitch_Max  0x03

#define EEPROM_Num_WP     0x04
#define EEPROM_Crnt_WP    0x05
#define EEPROM_WP_Radius  0x06
#define EEPROM_FBW_Throttle   0x07

#define EEPROM_AirSp_Bias 0x08
#define EEPROM_Launch_Alt 0x0A
#define EEPROM_Max_Alt    0x0C
#define EEPROM_Max_Speed  0x0E

#define EEPROM_Roll_Kp   0x14
#define EEPROM_Roll_Ki   0x16
#define EEPROM_Pitch_Kp  0x18
#define EEPROM_Pitch_Ki  0x1A

#define EEPROM_Roll_Trim   0x1C
#define EEPROM_Pitch_Trim  0x1E

#define First_WP_Byte      0x20      // First WP byte in EEPROM 

#define kRoll  0
#define kPitch 1

/***************************************************************************
 General variables
 **************************************************************************/
 
/*
float kp[] = {roll_P, pitch_P},
      ki[] = {roll_I, pitch_I}; 
*/

float kp[2], ki[2];     // Kp, Ki values for PID routines.

// -------------------------------------
// union is used to convert signed words to unsigned ints

union word_union {
  int16_t word;
  uint16_t usWord;
} wordUnion;


//GPS stuff please read SiRF-Binary-Protocol-Reference-Manual page 87 for more information
union long_union {
	int32_t dword;
	uint8_t  byte[4];
} longUnion;

union int_union {
	int16_t word;
	uint8_t  byte[2];
} intUnion;

// -------------------------------------

/*Flight GPS variables*/
int gpsFix = NO_FIX;       // Status of the last GPS message
float lat=0;               // store the Latitude from the gps
float lon=0;
float alt_MSL=0;           // This is the alt.
float ground_speed=0;      // This is the velocity your "plane" is traveling in meters for second, 1Meters/Second= 3.6Km/H = 1.944 knots
float ground_course=0;     // This is the runaway direction of you "plane" in degrees
float climb_rate=0;        // This is the verticle velocity in meters for seconds
char data_update_event=0; 

unsigned long PID_timer=0;
unsigned long NAV_timer=0;

float analog0 = 511; // Roll Analog ir sensor  
float analog1 = 511; // Pitch Analog ir sensor
float analog2 = 511; // Z Analog ir sensor
float analog3 = 224; // Airspeed (presure) sensor

int max_ir = 0;
int air_speed_offset = 224;
int air_speed_hold = 0 ;
int fly_by_wire_air_speed_hold = FLY_BY_WIRE_SPEED_SETPOINT;
int fly_by_wire_throttle = 40;

byte throttle_set_point=0;
int roll_set_point=0; //Stores the desired set point of roll and pitch 
int pitch_set_point=0;
unsigned int hold_Alt=0; 
unsigned int rx_Ch[2];

unsigned int wp_distance=0; 
unsigned int wp_bearing=0;
byte AP_Mode = 0;    // 0 = Manual; 1 = WP Mode; 2 = Return to Launch mode;  4 = Fly by wire

byte options; 
int roll_trim; 
int pitch_trim;
int roll_max, pitch_max;
int roll_fbw_gain, pitch_fbw_gain;

int max_alt = 0;
int max_spd = 0;

byte wp_number = 2; // Number of waypoints defined.... Go to to Waypoint tab...
byte current_wp=0;  // Store the current waypoint...
byte wp_radius=20;  // Radius of the waypoint, normally set as 20 meters.

unsigned int launch_alt=0; // Launch altitude

//Stores the actual waypoint we are trying to reach. 
float wp_current_lat=0;
float wp_current_lon=0;
unsigned int wp_current_alt=0; 

long refresh_rate=0;
const float t7=1000000.0;

/***************************************************************************
 NMEA variables
 **************************************************************************/

#if GPS_PROTOCOL == 0 // This condition is used by the compiler only....
/*GPS Pointers*/
char *token; //Some pointers
char *search = ",";
char *brkb, *pEnd;
char gps_buffer[100]; //The tradional buffer.
#endif

/***************************************************************************
 SIRD variables
 **************************************************************************/

#if GPS_PROTOCOL == 1

/*
 // $PSRF100,0,   57600,8,1,0    *37
 byte gps_buffer[90]={ 0x24,0x50,0x53,0x52,0x46,0x31,0x30,0x30,0x2C,0x30,0x2C,
  0x35,0x37,0x36,0x30,0x30,0x2C,0x38,0x2C,0x31,0x2C,0x30,
  0x2A,0x33,0x37,0x0D,0x0A}; 
*/

 // $PSRF100,0,   19200,8,1,0    *39
 byte gps_buffer[90]={ 0x24,0x50,0x53,0x52,0x46,0x31,0x30,0x30,0x2C,0x30,0x2C,
  0x31,0x39,0x32,0x30,0x30,0x2C,0x38,0x2C,0x31,0x2C,0x30,
  0x2A,0x33,0x39,0x0D,0x0A}; 
 
 const byte gps_ender[]={0xB0,0xB3};  //Used to configure Sirf GPS
#endif

/***************************************************************************
 uBlox Variables
 **************************************************************************/

#if GPS_PROTOCOL == 2
//uBlox Checksum
byte ck_a=0;
byte ck_b=0;
long iTOW=0;       // GPS Millisecond Time of Week
long alt=0;        // Height above Ellipsoid 
long speed_3d=0;   // Speed (3-D)  (not used)
#endif

/***************************************************************************
 **************************************************************************/

void setup()
{
  init_ardupilot();  // Initialize ardupilot...
}

// ---------------------------------

void loop()  // Main Loop
{
  navigation();    // Calculate all the setpoints for the roll, pitch and throttle. Function localted in "Ar_Main" tab.
  stabilization(); // Control of all the actuators using IRs and Pressure Sensor... and the desired Setpoints for each one.

  catch_analogs();    // Read and average the IR sensors.
  decode_gps();       // Decode GPS data
  check_XBee_data();  // Msgs from Gnd Station
  
  print_data();       // Output data to ground station.  Function located in "System" tab.
  
  read_Ch1();         // read the R/C Xmitter Stick positions.
  read_Ch2();
  
  refresh_rate++;
}

// ---------------------------------
